program pl_mp_client_web_app;

{$mode objfpc}{$H+}

uses
  {$IFDEF UNIX}
  cthreads,
  {$ENDIF}
  Classes, SysUtils, lnetbase, CustApp,
  fpjson, jsonscanner, jsonparser, Crt,
  WebSocket2, uplWebSocket
  { you can add units after this };

type


  { TPL_MP_Client_WebApp }
  TPL_MP_Client_WebApp = class(TCustomApplication)
  protected
    procedure DoRun; override;
  public
    WebSocketServer:TPLWebSocketServer;

    quit, noKeyPress:boolean;
    IsOKCommand:Boolean;

    AppDir, ConfigDir, ConfigFile:String;
    MyHost, MyWebHost:String;
    MyPort, MyWebPort:Integer;
    ClientName, ClientPlace, AudioLen:String;

    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;

    procedure LoadFromConfigFile(const aConfigFile:string);
    procedure CheckParameter();

    procedure SendMessageOnClient(const aJObject:TJSONObject);
    procedure OnWebSocketData(const str:string);

    procedure SendMessage1(const aTargetDiv, aMessage:String);
  end;

var
  Application: TPL_MP_Client_WebApp;

{ TPL_MP_Client_WebApp }

procedure TPL_MP_Client_WebApp.DoRun;
begin
  if not quit then begin
    WebSocketServer.OnData:=@OnWebSocketData;
    WebSocketServer.Start(MyWebHost,MyWebPort);

    repeat
      if IsOKCommand then
          quit:=true;
      if noKeyPress then begin
        if Keypressed then begin
          case readkey of
            #27: quit := true;

            '1': begin
              writeln('1');
            end; // 1

            '2': begin
              writeln('2');
            end; // 2

            '3': begin
              ClrScr;
              writeln('3');
            end; // 3

            '4': begin
              writeln('4');
            end; // 4

            '5': begin
              writeln('5');
            end; // 5

            '6': begin
               writeln('6');
            end; // 6
          end;
        end;
      end;
     delay(1);
    until quit;
  end;
  FreeAndNil(WebSocketServer);
  Terminate;
end;

constructor TPL_MP_Client_WebApp.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  WebSocketServer:=TPLWebSocketServer.Create;
  IsOKCommand:=false;
  quit:=false;
  noKeyPress:=true;

  AppDir:=ExtractFileDir(ParamStr(0)) + DirectorySeparator;

  ConfigDir:=AppDir;
  ConfigFile:=ConfigDir + 'pl_mp_clientWebApp.json';

  ClientName:='Client-Web- ' + IntToStr(Random(1000));
  clientplace:='';
  myHost:='10.10.10.10';
  myPort:=5060;
  MyWebPort:=4010;

  LoadFromConfigFile(ConfigFile);
  CheckParameter();
end; // TPL_MP_Client_WebApp.Create

destructor TPL_MP_Client_WebApp.Destroy;
begin
  inherited Destroy;
end; // TPL_MP_Client_WebApp.Destroy

procedure TPL_MP_Client_WebApp.LoadFromConfigFile(const aConfigFile: string);
var
  FileStream:TFileStream;
  JParser:TJSONParser;
  JOBject:TJSONObject;
//  JData:TJSONData;
begin
  if FileExists(aConfigFile) then begin
    FileStream:=TFileStream.Create(aConfigFile, fmOpenRead);
    JParser:=TJSONParser.Create(FileStream,[joIgnoreTrailingComma]);
    JOBject:=JParser.Parse as TJSONObject;

    MyHost:=JOBject.Elements['Horst'].AsString;
    MyWebHost:=JOBject.Elements['WebHost'].AsString;

    MyPort:=JOBject.Elements['Port'].AsInteger;
    MyWebPort:=JOBject.Elements['WebPort'].AsInteger;

    ClientName:=JOBject.Elements['ClientName'].AsString;
    ClientPlace:=JOBject.Elements['Place'].AsString;

//    JData:=JOBject.Find('CBConnect');
//    if Assigned(JData) then
//      CBConnect.Checked:=JData.AsBoolean;

    FreeAndNil(JParser); FreeAndNil(FileStream);
  end;

end; // TPL_MP_Client_WebApp.LoadFromConfigFile

procedure TPL_MP_Client_WebApp.CheckParameter();
var
  Token:string;
  i:Integer;
begin
  quit:=false; noKeyPress:=true;
  Token:='';

  if Paramcount > 0 then begin
    for i:=1 to Paramcount do begin
      Token:=LowerCase(ParamStr(i));
      if Token = 'nokeypress' then noKeyPress:=false;
      if Token = 'host' then myHost:=ParamStr(i + 1);
      if Token = 'web-host' then MyWebHost:=ParamStr(i + 1);
      if Token = 'port' then myPort:=StrToInt(ParamStr(i + 1));
      if Token = 'web-port' then MyWebPort:=StrToInt(ParamStr(i + 1));

      if Token = 'clientname' then ClientName:=ParamStr(i + 1);
      if Token = 'clientplace' then ClientPlace:=ParamStr(i + 1);
    end; // for i
  end;

end;


procedure TPL_MP_Client_WebApp.SendMessageOnClient(const aJObject: TJSONObject);
begin
  WebSocketServer.SendAText(aJObject.FormatJSON([]));
end;

procedure TPL_MP_Client_WebApp.OnWebSocketData(const str: string);
begin
  writeln(#13#10, 'TPL_MP_Client_WebApp.OnWebSocketData');
//  with MP_ClientAPI.RequestClass do begin
//    SendCommondList([Toggle(-1)]);
//  end;
end; // TPL_MP_Client_WebApp.OnWebSocketData

procedure TPL_MP_Client_WebApp.SendMessage1(const aTargetDiv, aMessage: String);
var
  JObject:TJSONObject;
begin
  JObject:=TJSONObject.Create;
  JObject.Add('TargetDiv',aTargetDiv);
  JObject.Add('Content',aMessage);

  SendMessageOnClient(JObject);
end; // TPL_MP_Client_WebApp.SendMessage1

begin
  Application:=TPL_MP_Client_WebApp.Create(nil);
  Application.Title:='Plutos MediaPlayer Web Interface ';
  Application.Run;
  Application.Free;
end.

