{
  Notiert:
  http://wiki.selfhtml.org/wiki/JavaScript/Objekte/DOM/document/forms/elements

  www.bennyn.de/programmierung/java/html5-websocket-server-mit-java-und-client-mit-javascript.html
}

unit uplWebSocket;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, synautil, fpjson,
  blcksock, customserver2, WebSocket2, synachar,
  uplthreadtimertest;

type

  TPLONWebSocketServer_Reaad = procedure (const str, from:String) of object;
  TPLONWebServer_Data = procedure(const str:string) of object;

  { TPLWebSocketServerConnection }
  TPLWebSocketServerConnection = class(TWebSocketServerConnection)
  private

  protected

  public
    CurrentDirectory:TJSONObject;

    constructor Create(aSocket: TTCPCustomConnectionSocket); override;
    destructor Destroy; override;
  published
  end; // TPLWebSocketServerConnection

  { TWebSocketServer2 }
  TWebSocketServer2 = class(TWebSocketServer)
    protected
      function GetWebSocketConnectionClass(): TWebSocketServerConnections; override;
    public
      procedure RemoveConnection(aConnection: TCustomConnection);
  end;

  { TPLWebSocketServer }
  TPLWebSocketServer = class
  private
    fOnData: TPLONWebServer_Data;
    fONWebSocketServer_Reaad: TPLONWebSocketServer_Reaad;
  protected
    function MusikDBToHtml(const aJObject:TJSONObject):string;

  public
    AppDir:string;
    SaveHost:string;
    Server:TWebSocketServer2;
    SavePort:Integer;
    Error:Boolean;
    plThreadTimer:TPLThreadTimer;
    LastUpdate:Integer;
    WebSite,StrListA:TStringList;

    constructor Create;
    destructor Destroy; override;

    procedure ResourceLoad(const aResource:string; aSocket: TTCPCustomConnectionSocket);

    function Start(Host:String; port:Integer):boolean;
    procedure OnAfterAddConnection(aServer: TCustomServer; aConnection: TCustomConnection);
    procedure OnBeforeAddConnection(aServer: TCustomServer; aConnection: TCustomConnection; var CanAdd: boolean);
    procedure OnAfterRemoveConnection(aServer: TCustomServer; aConnection: TCustomConnection);
    procedure OnBeforeRemoveConnection(aServer: TCustomServer; aConnection: TCustomConnection);
    procedure OnServerSocketError(aServer: TCustomServer; Socket: TTCPBlockSocket);

    procedure OnOpen(aSender: TWebSocketCustomConnection);
    procedure OnRead(aSender: TWebSocketCustomConnection; aFinal, aRes1, aRes2, aRes3: boolean; aCode: integer; aData: TMemoryStream);

    procedure OnWrite(aSender: TWebSocketCustomConnection; aFinal, aRes1, aRes2, aRes3: boolean; aCode: integer; aData: TMemoryStream);
    procedure OnClose(aSender: TWebSocketCustomConnection; aCloseCode: integer; aCloseReason: string; aClosedByPeer: boolean);
    procedure OnConnectionSocket(Sender: TObject; Reason: THookSocketReason; const Value: String);
    procedure OnHookHeartbeat(Sender:TObject);


    procedure SendAText(const aText:String);

    procedure TextOut(Line:integer; const ID, Value, Einheit:String);
    procedure TimerEvent;

  published
    property ONWebSocketServer_Reaad:TPLONWebSocketServer_Reaad read fONWebSocketServer_Reaad write fONWebSocketServer_Reaad;
    property OnData:TPLONWebServer_Data read fOnData write fOnData;


  end; // TPLWebSocketServer

var
  TestServer:TPLWebSocketServer;

implementation

uses math;


{ TPLWebSocketServer }
function ReadFileStream(f:TFileStream):string;
var
  ch:Char;
  str:String;
  r:integer;
begin
  ch:=' ';
  result:='';
  if Assigned(f) then begin
    f.Position:=0;
    str:='';
    repeat
      r:=f.Read(ch,1);
      str:=str+ch;
      sleep(10);
    until r <= 0;
    result:=str;
  end;
end;

{ TPLWebSocketServerConnection }

constructor TPLWebSocketServerConnection.Create(aSocket: TTCPCustomConnectionSocket);
begin
  inherited Create(aSocket);
  CurrentDirectory:=nil;
end; // TPLWebSocketServerConnection.Create

destructor TPLWebSocketServerConnection.Destroy;
begin
  inherited Destroy;
end; // TPLWebSocketServerConnection.Destroy

function TWebSocketServer2.GetWebSocketConnectionClass():TWebSocketServerConnections;
begin
  Result:=TPLWebSocketServerConnection;
end; // TWebSocketServer2.GetWebSocketConnectionClass

{ TWebSocketServer2 }
procedure TWebSocketServer2.RemoveConnection(aConnection: TCustomConnection);
begin
  inherited RemoveConnection(aConnection);
end; // TWebSocketServer2.RemoveConnection

function TPLWebSocketServer.MusikDBToHtml(const aJObject: TJSONObject): string;
var
  i:Integer;
  JObject:TJSONObject;
  jArray:TJSONArray;
  StrItem, TempString:String;
begin
  jArray:=aJObject.Elements['Items'] as TJSONArray;
  result:='';
  for i:=0 to jArray.Count - 1 do begin
    JObject:=jArray[i] as TJSONObject;
    TempString:=JObject.Elements['RelativeDirectory'].AsString;

    StrItem:=format('<A HREF="%s">', [TempString]);
    StrItem:=StrItem + TempString;
    StrItem:=StrItem + '/';
    result:=result + StrItem + '<br>';
  end;
end; // TPLWebSocketServer.MusikDBToHtml

constructor TPLWebSocketServer.Create;
begin
  inherited Create;
  TestServer:=self;
  StrListA:=TStringList.Create;
  WebSite:=TStringList.Create;

  AppDir:=ExtractFileDir(ParamStr(0))+DirectorySeparator;
  Error:=False;
  Server:=nil;
  fONWebSocketServer_Reaad:=nil;

  plThreadTimer:=TPLThreadTimer.Create(True);
  plThreadTimer.Interval:=1000;
  plThreadTimer.OnTimerEvent:=@TimerEvent;
  plThreadTimer.Enabled:=false;
  fOnData:=nil;
end; // TPLWebSocketServer.Create

// https://libwebsockets.org/lws-api-doc-master/html/group__wsclose.html#ggae399c571df32ba532c0ca67da9284985a9737a68759e739856b150ff9dfa30218
// https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent#Status_codes
destructor TPLWebSocketServer.Destroy;
begin
  FreeAndNil(plThreadTimer);
  WebSite.Free; StrListA.Free;
  if Assigned(Server) then begin
    Server.CloseAllConnections(wsCloseNormal,'Server beendet !');
    Server.Terminate;
    FreeAndNil(Server);
  end;
  inherited Destroy;
end; // TPLWebSocketServer.Destroy

procedure TPLWebSocketServer.ResourceLoad(const aResource: string; aSocket: TTCPCustomConnectionSocket);
var
  Header, OutputDataString, ext, contentType:string;
  StrList:TStrings;
  MS,MS2:TMemoryStream;

  NewStr:String;
  i:Integer;
begin
//  writeln(#13,'TPLWebSocketServer.ResourceLoad: "',AppDir+'module'+aResource,'"');
  try
    if aResource = '/' then begin
      WebSite.LoadFromFile(AppDir+'module/index.html');

      NewStr:=format('socket = new WebSocket(''ws://%s:%d'');',[SaveHost,SavePort]);
      for i:=0 to WebSite.Count-1 do begin
        if Pos('socket = new WebSocket(',WebSite[i]) > 0 then begin
          WebSite[i]:=NewStr;
        end;
      end;
      OutputDataString:=WebSite.Text;

      Header:='HTTP/1.0 200 '+#13#10+
              'Content-type: Text/Html; charset=UTF-8' + #13#10 +
              'Content-length: ' +inttostr(Length(OutputDataString))+#13#10+
              'Connection: close' + #13#10 + #13#10;
      aSocket.SendString(Header+OutputDataString);
    end // fncResourceName
    else begin
      ext:=lowercase(ExtractFileExt(aResource));
      contentType:='';
      if ext = '.js' then contentType:='text/javascript';
      if ext = '.map' then contentType:='text/map';
      if ext = '.css' then contentType:='text/css';
      if (ext = '.ico') or (ext = '.png') then contentType:='image/png';

      if contentType <> '' then begin
        if (FileExists(AppDir + 'module/' + aResource)) or (aResource = '/favicon.ico') then begin
          MS:=TMemoryStream.Create;
          MS2:=TMemoryStream.Create;
          if aResource = '/favicon.ico' then
            MS2.LoadFromFile(AppDir + 'module/favicon.png')
          else
            MS2.LoadFromFile(AppDir + 'module/' + aResource);

          Header:='HTTP/1.1 200' + #13#10+
                  'Content-type: ' + contentType + #13#10 +
                  'Content-length: '+ inttostr(MS2.Size) +#13#10 +
                  'Connection: close' + #13#10 + #13#10;

          MS.WriteBuffer(Pointer(Header)^,Length(Header));
          MS.CopyFrom(MS2,MS2.Size);
          MS.Position:=0;

          aSocket.SendStream(MS);
        //  sleep(100);
          MS.Free; MS2.Free;
          aSocket.SetLinger(true,0);
        end;
      end
      else begin
        write(#13, 'Datei "', AppDir + 'module/' + aResource, ' gibt es nicht !');
      end;
    end;

  except
    on E: Exception do
      writeln('TPLWebSocketServer.ResourceLoad:',E.Message);
  end;
end; // TPLWebSocketServer.ResourceLoad

function TPLWebSocketServer.Start(Host: String; port: Integer):boolean;
begin
  if Assigned(Server) then FreeAndNil(Server);
  SavePort:=Port;
  SaveHost:=Host;

  writeln('Host:',Host,' Port: ', Port);

  Server := TWebSocketServer2.Create(Host, IntToStr(SavePort));
  Server.OnResourceLoad:=@ResourceLoad;
  Server.OnAfterAddConnection := @OnAfterAddConnection;
  Server.OnBeforeAddConnection := @OnBeforeAddConnection;
  Server.OnAfterRemoveConnection := @OnAfterRemoveConnection;
  Server.OnBeforeRemoveConnection := @OnBeforeRemoveConnection;

  Server.OnSocketError := @OnServerSocketError;
  Server.SSL:=False;
  Server.Start;
  result:=false;
end; // TPLWebSocketServer.Start

procedure TPLWebSocketServer.OnAfterAddConnection(aServer: TCustomServer; aConnection: TCustomConnection);
begin
  TWebSocketServerConnection(aConnection).Socket.HeartbeatRate:=2000;
  TWebSocketServerConnection(aConnection).Socket.Family:=SF_IP4;
  TWebSocketServerConnection(aConnection).OnWrite := @OnWrite;
  TWebSocketServerConnection(aConnection).OnRead := @OnRead;
  TWebSocketServerConnection(aConnection).OnClose := @OnClose;
  TWebSocketServerConnection(aConnection).OnOpen := @OnOpen;
  TWebSocketServerConnection(aConnection).Socket.OnHeartbeat:=@OnHookHeartbeat;
end; // TPLWebSocketServer.OnAfterAddConnection

procedure TPLWebSocketServer.OnBeforeAddConnection(aServer: TCustomServer; aConnection: TCustomConnection; var CanAdd: boolean);
begin
  CanAdd:=true;
  TWebSocketServerConnection(aConnection).Socket.HeartbeatRate:=2000;

end; // TPLWebSocketServer.OnBeforeAddConnection

procedure TPLWebSocketServer.OnAfterRemoveConnection(aServer: TCustomServer; aConnection: TCustomConnection);
begin
  TWebSocketServerConnection(aConnection).OnWrite := nil;
  TWebSocketServerConnection(aConnection).OnRead := nil;
  TWebSocketServerConnection(aConnection).OnClose := nil;
  TWebSocketServerConnection(aConnection).OnOpen := nil;
end; // TPLWebSocketServer.OnAfterRemoveConnection

procedure TPLWebSocketServer.OnBeforeRemoveConnection(aServer: TCustomServer; aConnection: TCustomConnection);
begin
  TWebSocketServerConnection(aConnection).OnWrite := nil;
  TWebSocketServerConnection(aConnection).OnRead := nil;
  TWebSocketServerConnection(aConnection).OnClose := nil;
  TWebSocketServerConnection(aConnection).OnOpen := nil;
end; // TPLWebSocketServer.OnBeforeRemoveConnection

procedure TPLWebSocketServer.OnServerSocketError(aServer: TCustomServer; Socket: TTCPBlockSocket);
begin
  Error:=True;
{  if Socket.LastError = 98 then begin
    Socket.CloseSocket;
    Error:=False;

    FreeAndNil(Server);
    Start(SaveHost, SavePort);
  end
  else}
    writeln(Format('%s - %d (%s)', [FormatDateTime('yyyy-mm-dd hh:nn:ss', now), Socket.LastError, Socket.LastErrorDesc]));

    //Socket.Free;

  //  Start(SaveHost, SavePort);

end; // TPLWebSocketServer.OnServerSocketError

procedure TPLWebSocketServer.OnOpen(aSender: TWebSocketCustomConnection);
var
  PLWebSocketServerConnection:TPLWebSocketServerConnection;
begin
{  PLWebSocketServerConnection:=aSender as TPLWebSocketServerConnection;
  if Assigned(PLWebSocketServerConnection.CurrentDirectory) then begin

  end
  else begin

    SendAText(MusicDB_JObject.FormatJSON([]));
 end;}
end; // TPLWebSocketServer.OnOpen

procedure TPLWebSocketServer.OnRead(aSender: TWebSocketCustomConnection; aFinal, aRes1, aRes2, aRes3: boolean; aCode: integer; aData: TMemoryStream);
var
  str,s:string;
begin
  try
    aData.Position:=0;
    s := ReadStrFromStream(aData, min(aData.size, 10 * 1024));
    if (aCode = wsCodeText) then
      str := trim(CharsetConversion(s, ISO_8859_1, ISO_8859_1))
    else
      str:= trim(s);

    writeln(#13#10, '"',str,'"');
//    if (str = 'Toggle') then
  except
    on E: Exception do
      writeln('TPLWebSocketServer.OnRead:',E.Message);
  end;
end; // TPLWebSocketServer.OnRead

procedure TPLWebSocketServer.OnWrite(aSender: TWebSocketCustomConnection; aFinal, aRes1, aRes2, aRes3: boolean; aCode: integer; aData: TMemoryStream);
begin
//  writeln('onWrite');
end; // TPLWebSocketServer.OnWrite

procedure TPLWebSocketServer.OnClose(aSender: TWebSocketCustomConnection; aCloseCode: integer; aCloseReason: string; aClosedByPeer: boolean);
begin
  //writeln('OnClose: ', aCloseCode, ' ', aCloseReason, ' ', aClosedByPeer);
//  Server.RemoveConnection(aSender);
end; // TPLWebSocketServer.OnClose

procedure TPLWebSocketServer.OnConnectionSocket(Sender: TObject; Reason: THookSocketReason; const Value: String);
begin
  writeln('OnConnectionSocket');
end; // TPLWebSocketServer.OnConnectionSocket

procedure TPLWebSocketServer.OnHookHeartbeat(Sender: TObject);
begin
  writeln(Sender.ClassName);
end; // TPLWebSocketServer.OnHookHeartbeat

procedure TPLWebSocketServer.SendAText(const aText: String);
var
  i:integer;
begin
  try
    if (Assigned(Server)) and (Server.Count >=1) then begin
      Server.LockTermination;
      for I := 0 to Server.Count - 1 do
        TWebSocketCustomConnection(Server.Connection[i]).SendText(CharsetConversion(aText, GetCurCP, UTF_8));
      Server.UnLockTermination;
    end;
  except
    on E: Exception do
      writeln('TPLWebSocketServer.SendAText:',E.Message);
  end;
end;

procedure TPLWebSocketServer.TextOut(Line:integer; const ID, Value, Einheit:String);
begin
  try
    SendAText('2$'+Value+' ' + Einheit+'$'+ID);
  except
    on E: Exception do
      writeln('TPLWebSocketServer.TextOut:',E.Message);
  end;
end;

procedure TPLWebSocketServer.TimerEvent;
begin
{  try
//    inc(LastUpdate);
//    SendAText('3$'+intTostr(LastUpdate));
  except
    on E: Exception do
      writeln('TPLWebSocketServer.TimerEvent:',E.Message);
  end;}
end; // TPLWebSocketServer.TimerEvent


end.
